from typing import Any, List

from fastapi import APIRouter, Depends, HTTPException
from sqlalchemy.orm import Session

from app import crud
from app.api import deps
from app.schemas.supplier import Supplier, SupplierCreate, SupplierUpdate

router = APIRouter()


@router.get("/", response_model=List[Supplier])
def read_suppliers(
    db: Session = Depends(deps.get_db),
    skip: int = 0,
    limit: int = 100,
    current_user: Any = Depends(deps.get_current_active_user),
) -> Any:
    """
    Retrieve suppliers.
    """
    suppliers = crud.supplier.get_multi(db, skip=skip, limit=limit)
    return suppliers


@router.post("/", response_model=Supplier)
def create_supplier(
    *,
    db: Session = Depends(deps.get_db),
    supplier_in: SupplierCreate,
    current_user: Any = Depends(deps.get_current_active_user),
) -> Any:
    """
    Create new supplier.
    """
    supplier = crud.supplier.create(db, obj_in=supplier_in)
    return supplier


@router.put("/{id}", response_model=Supplier)
def update_supplier(
    *,
    db: Session = Depends(deps.get_db),
    id: int,
    supplier_in: SupplierUpdate,
    current_user: Any = Depends(deps.get_current_active_user),
) -> Any:
    """
    Update a supplier.
    """
    supplier = crud.supplier.get(db, id=id)
    if not supplier:
        raise HTTPException(status_code=404, detail="Supplier not found")
    supplier = crud.supplier.update(db, db_obj=supplier, obj_in=supplier_in)
    return supplier


@router.get("/{id}", response_model=Supplier)
def read_supplier(
    *,
    db: Session = Depends(deps.get_db),
    id: int,
    current_user: Any = Depends(deps.get_current_active_user),
) -> Any:
    """
    Get supplier by ID.
    """
    supplier = crud.supplier.get(db, id=id)
    if not supplier:
        raise HTTPException(status_code=404, detail="Supplier not found")
    return supplier


@router.delete("/{id}", response_model=Supplier)
def delete_supplier(
    *,
    db: Session = Depends(deps.get_db),
    id: int,
    current_user: Any = Depends(deps.get_current_active_superuser),
) -> Any:
    """
    Delete a supplier.
    """
    supplier = crud.supplier.get(db, id=id)
    if not supplier:
        raise HTTPException(status_code=404, detail="Supplier not found")
    
    # Check if supplier has any items
    items = crud.item.get_by_supplier(db, supplier_id=id)
    if items:
        raise HTTPException(
            status_code=400,
            detail="Cannot delete supplier with associated items. Remove the items first.",
        )
    
    supplier = crud.supplier.remove(db, id=id)
    return supplier